// Copyright 2008-2009  Segher Boessenkool  <segher@kernel.crashing.org>
// This code is licensed to you under the terms of the GNU GPL, version 2;
// see file COPYING or http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt

#include "loader.h"

static u8 *const code_buffer = (u8 *)0x90100000;
static u8 *const trampoline_buffer = (u8 *)0x80001800;
static u8 iself = 1;

static void MEM_UNPROT(void)
{
	write16(0x0C00401C,0);
	write16(0x0C004010,0xFFFF);
	write32(0x0C004000,0);
	write32(0x0C004004,0);
	write32(0x0C004008,0);
	write32(0x0C00400C,0);
}
static void dsp_reset(void)
{
	write16(0x0c00500a, read16(0x0c00500a) & ~0x01f8);
	write16(0x0c00500a, read16(0x0c00500a) | 0x0010);
	write16(0x0c005036, 0);
}

int try_sd_load(void)
{
	int err;

	err = sd_init();
	if (err) {
	
		printf("SD card not found (%d)\n", err);
		return err;
		dbvideocollor(0x4c544cff); // red
	}

	err = fat_init();
	if (err == 0)
		printf("SD card detected\n");
	else {
		dbvideocollor(0x40804080); // grey
		printf("SD card not detected (%d)\n", err);
		return err;
	}

	printf("Opening boot.elf:\n");
	err = fat_open("boot.elf");
	


	if (err) {
		printf("Opening boot.dol:\n");
		err = fat_open("boot.dol");
		iself = 0;
	}

extern u32 fat_file_size;

	printf("reading %d bytes...\n", fat_file_size);
	err = fat_read(code_buffer, fat_file_size);
	if (err) {
		dbvideocollor(0x266a26c0); // maroon
		printf("Error %d reading file\n", err);
		return err;
	}

	printf("Done.\n");
	return 0;
}

int try_usbgecko_load(void)
{
	if (!usbgecko_checkgecko()) {
		printf("USBGecko not found\n");
		return -1;
	}

#define FLASH_OFFSET 0x30000
	int i, size = usbgecko_flash_read32(FLASH_OFFSET);
	if (size < 0) {
		printf("Invalid code size in usbgecko flash (%d)\n", size);
		return -1;
	}
	printf("Loading %d bytes from USBGecko flash (offset=%x)\n",
		size, FLASH_OFFSET+4);

	for (i=0; i < size; i++)
		code_buffer[i] = usbgecko_flash_read8(FLASH_OFFSET + 4 + i);

	return 0;
}

int maino(void)
{
	*(u32*)0xCD0000C0 |= 0x20;
	dsp_reset();
	//mfspr


	// Clear interrupt mask.
	write32(0x0c003004, 0);
	exception_init(); //hope it work
	// Unlock EXI.
	write32(0x0d00643c, 0);
	video_init();
	MEM_UNPROT();
	usbgecko_init();
	printf("savezelda\nmod for YU-GI-OH by ichfly\nCleaning up environment...");

	reset_ios();
	
	printf("OK.\n");


	int err;
	
	dbvideocollor(0x7140718a); // olive
 restart:
	err = try_sd_load();

	if (err) {
	
		dbvideocollor(0x552b5515); // lime
		err = try_usbgecko_load();

		if (err) {
			printf("No code found to load, hanging.\n");
			for (;;)
				;
		}
	}

	if (valid_elf_image(code_buffer)) {
		printf("Valid ELF image detected.\n");
		void (*entry)() = load_elf_image(code_buffer);
		dbvideocollor(0x00800080);
		*(u32*)0xCD0000C0 &= ~0x20;
		entry();
	} else
	{
		printf("No Valid ELF image detected.\n");
		if(iself == 0)
		{
			if (valid_dol_image(code_buffer))
			{
				printf("Valid DOL image detected.\n");
				void (*entry)() = load_dol_image(code_buffer);
				dbvideocollor(0x00800080);
				*(u32*)0xCD0000C0 &= ~0x20;
				entry();
			}
		}
	}
	printf("No valid ELF/DOL image detected, retrying.\n");

	dbvideocollor(0xc399c36a); // sky blue

	goto restart;
}
